/* ioctls.c: ioctl type decoder */

#include <sys/types.h>
#include <linux/ioctl.h>
#include <linux/soundcard.h>
#include <linux/cdrom.h>
#include "ioctls.h"

int prepare_ioctl_info(int request, int *props, int *size)
{
  switch (_IOC_TYPE(request)) {
    /* sound ioctls follow standard numbering scheme */
    case 'Q': case 'm': 
    case 'P': case 'C': 
    case 'M':
      *props = 0;
      if (_IOC_DIR(request) & _IOC_WRITE) *props |= PROPS_WRITE;
      if (_IOC_DIR(request) & _IOC_READ)  *props |= PROPS_READ;
      *size = _IOC_SIZE(request);
      return 1;
    
    /* we want to make an effort for cdrom ioctls */
    case 'S':
      switch (request) {
        case CDROMPAUSE:
        case CDROMRESUME:
        case CDROMSTOP:
        case CDROMSTART:
        case CDROMEJECT:
        case CDROMRESET:
        case CDROMCLOSETRAY:
          *props = 0; 
          *size = 0;
          return 1;
          
        case CDROMPLAYMSF:
          *props = PROPS_WRITE;
          *size = sizeof (struct cdrom_msf);
          return 1;

        case CDROMPLAYTRKIND:
          *props = PROPS_WRITE;
          *size = sizeof (struct cdrom_ti);
          return 1;

        case CDROMREADTOCHDR:
          *props = PROPS_READ;
          *size = sizeof (struct cdrom_tochdr);
          return 1;

        case CDROMREADTOCENTRY:
          *props = PROPS_READ;
          *size = sizeof (struct cdrom_tocentry);
          return 1;
          
        case CDROMVOLCTRL:
          *props = PROPS_WRITE;
          *size = sizeof (struct cdrom_volctrl);
          return 1;

        case CDROMVOLREAD:
          *props = PROPS_READ;
          *size = sizeof (struct cdrom_volctrl);
          return 1;
          
        case CDROMSUBCHNL:
          *props = PROPS_READ|PROPS_WRITE;
          *size = sizeof (struct cdrom_subchnl);
          return 1;
          
        case CDROMMULTISESSION:
          *props = PROPS_READ|PROPS_WRITE;
          *size = sizeof (struct cdrom_multisession);
          return 1;
          
        case CDROM_GET_MCN:
          *props = PROPS_READ;
          *size = sizeof (struct cdrom_mcn);
          return 1;
        
        case CDROMEJECT_SW:
        case CDROMREADMODE2:
        case CDROMREADMODE1:
        case CDROMREADAUDIO:
        case CDROMREADRAW:
          return 0; /* unsupported so far */
          
        default: return 0;
      }
    
    /* all others will remain massively unsupported for now */
    default: 
      return 0;
  }
}
